# Note: this file has been initially generated by Module::Build::Convert 0.47_02

use 5.008;
use strict;
use warnings;

use Module::Build;

my $build = Module::Build->new
  (
   module_name => 'Hook::Output::File',
   dist_author => 'Steven Schubiger <schubiger@cpan.org>',
   dist_version_from => 'lib/Hook/Output/File.pm',
   requires => {
                 'boolean' => 0,
                 'perl' => 5.008000,
                 'Carp' => 0,
                 'Cwd' => 0,
                 'IO::Handle' => 0,
                 'Params::Validate' => 0,
                 'Scalar::Util' => 0,
                 'Tie::Handle' => 0,
               },
   build_requires => {
                       'File::Temp' => 0,
                       'IO::Capture' => 0,
                       'Test::More' => 0,
                     },
   license => 'perl',
   create_readme => 1,
   create_makefile_pl => 'traditional',
  );

$build->create_build_script;
